#include "RTC_EEPROM.h"
#include "globals.h"
#include "pins.h"

/* -------------------------------------------------------------------
 *  Initialisiert den ATmega zum Betrieb des RTC/EEPROM-Moduls.
 */
void RTC_EEPROM_init(void) {
  pinMode(RTC_EEPROM_POWER_PIN, OUTPUT);
  Wire.begin();
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);  // RTC aus
}


/* -------------------------------------------------------------------
 *  Liefert Datum und Uhrzeit von der RTC in folgendem Format:
 *    Jahr(0-63) Monat(1-12) Tag(1-31) Stunde(0-23) Minute(0-59) Sekunde(0-59)
 *    6 Bit      4 Bit       5 Bit     5 Bit        6 Bit        6 Bit
 *    xxxxxx xxxx xxxxx xxxxx xxxxxx xxxxxx
 *    777766 6655 55444 43333 222211 110000
 */
/*unsigned long getRTC2(void) {
  unsigned long t=0;

  digitalWrite(RTC_EEPROM_POWER_PIN, HIGH);
  Wire.beginTransmission(DS3231_ADR);
  Wire.write(0);                // Register(0) (Seconds)
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADR, 7);     // 7 Byte holen
  t=lbcd2dec(Wire.read())&0x3F;        // Seconds
  t|=(lbcd2dec(Wire.read())&0x3F)<<6;  // Minutes
  t|=(lbcd2dec(Wire.read())&0x1F)<<12; // Hours
  Wire.read();                         // Day weglesen
  t|=(lbcd2dec(Wire.read())&0x1F)<<17; // Date
  t|=(lbcd2dec(Wire.read())&0x0F)<<22; // Month
  t|=(lbcd2dec(Wire.read())&0x3F)<<26; // Year
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);
  return(t);
}*/


/* -------------------------------------------------------------------
 *  Liefert Datum und Uhrzeit von der RTC als RTCdata.
 */
RTCdata getRTC(void) {
  RTCdata t;

  digitalWrite(RTC_EEPROM_POWER_PIN, HIGH);
  Wire.beginTransmission(DS3231_ADR);
  Wire.write(0);                      // Register(0) (Seconds)
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADR, 7);    // 7 Byte holen
  t.second=bcd2dec(Wire.read())&0x3F; // Seconds
  t.minute=bcd2dec(Wire.read())&0x3F; // Minutes
  t.hour  =bcd2dec(Wire.read())&0x1F; // Hours
  Wire.read();                        // Day weglesen
  t.day   =bcd2dec(Wire.read())&0x1F; // Date
  t.month =bcd2dec(Wire.read())&0x0F; // Month
  t.year  =bcd2dec(Wire.read())&0x3F; // Year
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);
  return(t);
}


/* -------------------------------------------------------------------
 *  Liefert die Sekunde von der RTC.
 */
byte getRTCSec(void) {
  byte t;

  digitalWrite(RTC_EEPROM_POWER_PIN, HIGH);
  Wire.beginTransmission(DS3231_ADR);
  Wire.write(0);                    // Register(0) (Seconds)
  Wire.endTransmission();
  Wire.requestFrom(DS3231_ADR, 1);  // 7 Byte holen
  t=bcd2dec(Wire.read())&0x3F;      // Seconds
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);
  return(t);
}


/* -------------------------------------------------------------------
 *  Debug-Print der Zeit "t".
 */
#if DEBUG==1
void printRTC(RTCdata t) {
  Serial.print(2000+t.year); Serial.print(".");  Serial.print(t.month);   Serial.print(".");  Serial.print(t.day);  Serial.print(" ");
  Serial.print(t.hour, DEC); Serial.print(":");  Serial.print(t.minute, DEC);  Serial.print(":");  Serial.print(t.second, DEC);
}
#endif

/* -------------------------------------------------------------------
 *  Debug-Print der Zeit "t".
 */
/*void printRTC2(unsigned long t) {
  Serial.print((t>>26)&0x3F);   Serial.print(".");
  Serial.print((t>>22)&0x0F);   Serial.print(".");
  Serial.print((t>>17)&0x1F);   Serial.print(" ");
  Serial.print((t>>12)&0x1F);   Serial.print(":");
  Serial.print((t>>6)&0x3F);    Serial.print(":");
  Serial.println(t&0x3F);
}*/


/* -------------------------------------------------------------------
 *  Stellt die RTC gemäß Parameter ein.
 */
void setRTC(byte YY, byte MM, byte DD, byte hh, byte mm, byte ss) {
  Wire.beginTransmission(DS3231_ADR);
  Wire.write(0);
  Wire.write(dec2bcd(ss));
  Wire.write(dec2bcd(mm));
  Wire.write(dec2bcd(hh));
  Wire.write(0);
  Wire.write(dec2bcd(DD));
  Wire.write(dec2bcd(MM));
  Wire.write(dec2bcd(YY));
  Wire.endTransmission();
}
/* -------------------------------------------------------------------
 * Schreibt einen String mit Datum+Zeit in die RTC.
 * Der String muss folgendes Format haben: YYYY.MM.DD hh:mm:ss
 */
void setRTC(char *datetime_str) {
  // Einzelstrings aus Timestamp bauen
  // 2015.10.20 19:48:08
  // 01234567890123456789
  //     X  X  X  X  X
  *(datetime_str+4)='\0';
  *(datetime_str+7)='\0';
  *(datetime_str+10)='\0';
  *(datetime_str+13)='\0';
  *(datetime_str+16)='\0';
  setRTC(atoi(datetime_str)-2000, atoi(datetime_str+5), atoi(datetime_str+8), 
          atoi(datetime_str+11), atoi(datetime_str+14), atoi(datetime_str+17));
}

/* -------------------------------------------------------------------
 *  Kleine Helferchen für RTC-Funktionen.
 */
byte dec2bcd(byte val) {  return((val/10*16)+(val%10));  }
byte bcd2dec(byte val) {  return((val/16*10)+(val%16));  }
unsigned long lbcd2dec(byte val) {  return((val/16*10)+(val%16));  }




/* ###################################################################
 * Es folgen die Funktionen für das EEPROM.
 */


/* -------------------------------------------------------------------
 *  Debug-Ausgabe
 */
#if DEBUG==1
void dumpEEPROM(void) {
  unsigned char buf[16];
  char c[7], j;

  for(int i=0; i<4096; i+=16) {         // über das gesamte EEPROM
    snprintf(c, 7, "%04x  ", i);        // Adresse ausgeben
    Serial.print(c);
    readEEPROM(i, buf, 16);             // 16 Byte lesen
    for(j=0; j<16; j++) {
      snprintf(c, 4, " %02x", buf[j]);  // 16 HEX-Werte ausgeben
      Serial.print(c);
    }
    Serial.print("  ");
    for(j=0; j<16; j++) {               // 16 ASCII-Werte
      if(buf[j]>=32 && buf[j]<128) {    // nur darstellbare Zeichen...
        snprintf(c, 2, "%c", buf[j]);
        Serial.print(c);                // ...ausgeben
      } else {
        Serial.print(".");              // sonst stattdessen "." ausgeben
      }
    }
    Serial.println();
  }
}
#endif

/* -------------------------------------------------------------------
 *  Schreibt "lng" Byte von "buf" an die EEPROM-Adresse ab "adr".
 *  Es wird in 16 Byte langen Blöcken geschrieben.
 */
byte writeEEPROMfull(unsigned int adr, unsigned char *buf, int lng) {
  byte rc;
  //Serial.print("writeEEPROMfull("); Serial.print(adr, HEX); Serial.print(", , "); Serial.print(lng); Serial.println(")");
  digitalWrite(RTC_EEPROM_POWER_PIN, HIGH);
  for(int i=lng, j=0; i>0; i-=16, j+=16) {
    if((rc=writeEEPROM(adr+j, buf+j, min(16, i)))!=0) {
      break;
    }
  }
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);
  return(rc);
}


/* -------------------------------------------------------------------
 *  Schreibt "lng" Byte (lng<=16) von "buf" an die EEPROM-Adresse 
 *  ab "adr".
 */
byte writeEEPROM(unsigned int adr, unsigned char *buf, byte lng) {
  byte rc;
  Wire.beginTransmission(AT24C32_ADR);
  Wire.write((adr>>8)&0xFF);
  Wire.write(adr&0xFF);
  Wire.write(buf, lng);
  rc=Wire.endTransmission();
  delay(5); // etwas Zeit zum Schreiben geben...
  return(rc);
}


/* -------------------------------------------------------------------
 *  Liest "lng" Byte ab der EEPROM-Adresse "adr" nach "buf".
 *  Es wird in 16 Byte langen Blöcken gelesen.
 */
void readEEPROMfull(unsigned int adr, unsigned char *buf, int lng) {
  for(int i=lng, j=0; i>0; i-=16, j+=16) {
    readEEPROM(adr+j, buf+j, min(16, i));
  }
}


/* -------------------------------------------------------------------
 *  Liest "lng" Byte (lng<=16) ab der EEPROM-Adresse "adr" nach "buf".
 */
void readEEPROM(unsigned int adr, unsigned char *buf, byte lng) {
  digitalWrite(RTC_EEPROM_POWER_PIN, HIGH);
  Wire.beginTransmission(AT24C32_ADR);
  Wire.write((adr>>8)&0xFF);
  Wire.write(adr&0xFF);
  Wire.endTransmission();
  Wire.requestFrom((uint8_t)AT24C32_ADR, (uint8_t)lng);
  for(byte i=0; i<lng; i++) {
    if(Wire.available()) {
      *buf++=Wire.read();
    }
  }
  digitalWrite(RTC_EEPROM_POWER_PIN, LOW);
}

